// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
// SPDX-License-Identifier: BSD-3-Clause
#include "vtkCellGridShaderBases.h"

const char *vtkCellGridShaderBases =
"// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen\n"
"// SPDX-License-Identifier: BSD-3-Clause\n"
"\n"
"// NB: The functions named NoneXxxI0_basisAt() are dummy functions used when\n"
"// not coloring by a field. They are never called.\n"
"\n"
"// Forward-declare the \"generic\" shape gradient so that specializations of\n"
"// {{ShapeName}}_normalToSideAt() can call it. This way things will just\n"
"// work, even with curved elements.\n"
"void shapeGradientAt(in vec3 param, in float coeff[{ShapeCoeffPerCell}], out vec3 dxdr, out vec3 dxds, out vec3 dxdt);\n"
"\n"
"/** Pyramidal function space interpolation */\n"
"#ifdef BASIS_HGradPyrI0\n"
"void HGradPyrI0_basisAt(in vec3 param, out float basis[1])\n"
"{{\n"
"  basis[0] = 1.0;\n"
"}}\n"
"\n"
"void HGradPyrI0_basisGradientAt(in vec3 param, out float basisGradient[3])\n"
"{{\n"
"  basisGradient[0] = 0.;\n"
"  basisGradient[1] = 0.;\n"
"  basisGradient[2] = 0.;\n"
"}}\n"
"#endif /* BASIS_HGradPyrI0 */\n"
"\n"
"#ifdef BASIS_HGradPyrC1\n"
"void HGradPyrC1_basisAt(in vec3 param, out float basis[5])\n"
"{{\n"
"  // constexpr float eps = std::numeric_limits<float>::epsilon();\n"
"  float eps = 1.19209e-07;\n"
"\n"
"  float tt = param.z;\n"
"  if (abs(tt - 1.0) < eps)\n"
"  {{\n"
"    if (tt <= 1.0) {{ tt = 1.0 - eps; }}\n"
"    else {{ tt = 1.0 + eps; }}\n"
"  }}\n"
"\n"
"  float ttTerm = 0.25/(1.0 - tt);\n"
"  basis[0] = (1.0 - param.x - tt) * (1.0 - param.y - tt) * ttTerm;\n"
"  basis[1] = (1.0 + param.x - tt) * (1.0 - param.y - tt) * ttTerm;\n"
"  basis[2] = (1.0 + param.x - tt) * (1.0 + param.y - tt) * ttTerm;\n"
"  basis[3] = (1.0 - param.x - tt) * (1.0 + param.y - tt) * ttTerm;\n"
"  basis[4] = tt;\n"
"}}\n"
"\n"
"void HGradPyrC1_basisGradientAt(in vec3 param, out float basisGradient[15]) // 5 * 3\n"
"{{\n"
"  // constexpr float eps = std::numeric_limits<float>::epsilon();\n"
"  float eps = 1.19209e-07;\n"
"\n"
"  float rr = param.x;\n"
"  float ss = param.y;\n"
"  float tt = param.z;\n"
"  // Be sure that the basis functions are defined when tt is very close to 1.\n"
"  // Warning: the derivatives are discontinuous in (0, 0, 1).\n"
"  if (abs(tt-1.0) < eps)\n"
"  {{\n"
"    if (tt <= 1.0) {{ tt = 1.0 - eps; }}\n"
"    else {{ tt = 1.0 + eps; }}\n"
"  }}\n"
"\n"
"  float ttTerm = 0.25/(1.0 - tt);\n"
"  float ttTerm2 = 4.0 * ttTerm * ttTerm;\n"
"\n"
"  basisGradient[ 0] = (ss + tt - 1.0) * ttTerm;\n"
"  basisGradient[ 1] = (rr + tt - 1.0) * ttTerm;\n"
"  basisGradient[ 2] = rr * ss * ttTerm2 - 0.25;\n"
"\n"
"  basisGradient[ 3] = (1.0 - ss - tt) * ttTerm;\n"
"  basisGradient[ 4] = (tt - rr - 1.0) * ttTerm;\n"
"  basisGradient[ 5] = rr * ss * ttTerm2 - 0.25;\n"
"\n"
"  basisGradient[ 6] = (1.0 + ss - tt) * ttTerm;\n"
"  basisGradient[ 7] = (1.0 + rr - tt) * ttTerm;\n"
"  basisGradient[ 8] = rr * ss * ttTerm2 - 0.25;\n"
"\n"
"  basisGradient[ 9] = (tt - ss - 1.0) * ttTerm;\n"
"  basisGradient[10] = (1.0 - rr - tt) * ttTerm;\n"
"  basisGradient[11] = rr * ss * ttTerm2 - 0.25;\n"
"\n"
"  basisGradient[12] = 0.0;\n"
"  basisGradient[13] = 0.0;\n"
"  basisGradient[14] = 1.0;\n"
"}}\n"
"#endif /* BASIS_HGradPyrC1 */\n"
"\n"
"#ifdef SHAPE_pyramid\n"
"int pyramid_axisPermutationForSide(in int side)\n"
"{{\n"
"  switch(side)\n"
"  {{\n"
"  case 0: return 2; // (+r, +t) = -S\n"
"  case 1: return 1; // (+s, +t) = +R\n"
"  case 2: return 3; // (-r, +t) = +S\n"
"  case 3: return 0; // (-s, +t) = -R\n"
"  case 4: return 4; // (-r, -s) = -T\n"
"  // case 5: return 5; // (+r, +s) = +T\n"
"  // case 1: return 6; // (-r+s,-r-s+t) = +RST\n"
"  // case 2: return 0; // (-s, +t) = -R\n"
"  // case 3: return 4; // (-r, -s) = -T\n"
"  }}\n"
"}}\n"
"\n"
"vec3 pyramid_normalToSideAt(in int sideId, in float shapeVals[{ShapeCoeffPerCell}], in vec3 rst, in vec3 cameraVector)\n"
"{{\n"
"  vec3 dxdr;\n"
"  vec3 dxds;\n"
"  vec3 dxdt;\n"
"  vec3 nn;\n"
"  shapeGradientAt(rst, shapeVals, dxdr, dxds, dxdt);\n"
"  vec3 corners[5];\n"
"  if (sideId < 4)\n"
"  {{\n"
"    for (int ii = 0; ii < 5; ++ii)\n"
"    {{\n"
"      corners[ii] = vec3(shapeVals[ii * 3], shapeVals[ii * 3 + 1], shapeVals[ii * 3 + 2]);\n"
"    }}\n"
"  }}\n"
"  switch (sideId)\n"
"  {{\n"
"  // Triangles\n"
"#if 1\n"
"     // NB: These work for a linear pyramid but will not compute normals for higher-order pyramids.\n"
"  case  0: nn = cross(corners[1] - corners[0], corners[4] - corners[0]); break;\n"
"  case  1: nn = cross(corners[2] - corners[1], corners[4] - corners[1]); break;\n"
"  case  2: nn = cross(corners[3] - corners[2], corners[4] - corners[2]); break;\n"
"  case  3: nn = cross(corners[0] - corners[3], corners[4] - corners[3]); break;\n"
"#else\n"
"     // NB: These would work in theory for higher-order pyramids, but fail due to a degeneracy in the parameter space.\n"
"  case  0: nn = cross( dxdr, 0.5 * ( dxdr + dxds) + dxdt); break; // -S -dxds + dxdt  cross(dxdr, dxdt)\n"
"  case  1: nn = cross( dxds, 0.5 * (-dxdr + dxds) + dxdt); break; // +R  dxdr + dxdt  cross(dxds, dxdt)\n"
"  case  2: nn = cross(-dxdr, 0.5 * (-dxdr - dxds) + dxdt); break; // +S  dxds + dxdt  cross(dxdt, dxdr)\n"
"  case  3: nn = cross(-dxds, 0.5 * ( dxdr - dxds) + dxdt); break; // -R -dxdr + dxdt  cross(dxdt, dxds)\n"
"#endif\n"
"  // Quadrilateral\n"
"  case  4: nn = cross(dxds, dxdr); break; // -T\n"
"  // Edges around base\n"
"  case  5: nn = cross(dxdr, dxdt); break; // Or cross(dxds, dxdr); // Edge is +R, norm -S or -T\n"
"  case  6: nn = cross(dxds, dxdt); break; // Or cross(dxds, dxdr); // Edge is +S, norm +R or -T\n"
"  case  7: nn = cross(dxdt, dxds); break; // Or cross(dxds, dxdr); // Edge is -R, norm +S or -T\n"
"  case  8: nn = cross(dxdt, dxdr); break; // Or cross(dxds, dxdr); // Edge is -S, norm -R or -T\n"
"  // Edges to tip\n"
"  case  9: nn = cross(dxdr,  dxds + 2 * dxdt); break; // Or cross(dxdr + 2 * dxdt, -dxds);\n"
"  case 10: nn = cross(dxds, -dxdr + 2 * dxdt); break; // Or cross(dxds + 2 * dxdt,  dxdr);\n"
"  case 11: nn = cross(dxdr, -dxds + 2 * dxdt); break; // Or cross(dxdr + 2 * dxdt,  dxds);\n"
"  case 12: nn = cross(dxds,  dxdr + 2 * dxdt); break; // Or cross(dxds + 2 * dxdt, -dxdr);\n"
"  // Base vertices\n"
"  case 13:\n"
"  case 14:\n"
"  case 15:\n"
"  case 16:\n"
"           nn = -dxdt; break;\n"
"  case 17: nn =  dxdt; break;\n"
"  default: nn = cameraVector; break;\n"
"  }}\n"
"  return normalize(nn);\n"
"}}\n"
"\n"
"void NonePyrI0_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"void NonePyrC1_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"#endif /* SHAPE_pyramid */\n"
"\n"
"/** Wedge function space interpolation */\n"
"#ifdef BASIS_HGradWdgI0\n"
"void HGradWdgI0_basisAt(in vec3 param, out float basis[1])\n"
"{{\n"
"  basis[0] = 1.0;\n"
"}}\n"
"\n"
"void HGradWdgI0_basisGradientAt(in vec3 param, out float basisGradient[3])\n"
"{{\n"
"  basisGradient[0] = 0.;\n"
"  basisGradient[1] = 0.;\n"
"  basisGradient[2] = 0.;\n"
"}}\n"
"#endif /* BASIS_HGradWdgI0 */\n"
"\n"
"#ifdef BASIS_HGradWdgC1\n"
"void HGradWdgC1_basisAt(in vec3 param, out float basis[6])\n"
"{{\n"
"  basis[0] = (1.0 - param.x - param.y)*(1.0 - param.z)/2.0;\n"
"  basis[1] = param.x*(1.0 - param.z)/2.0;\n"
"  basis[2] = param.y*(1.0 - param.z)/2.0;\n"
"  basis[3] = (1.0 - param.x - param.y)*(1.0 + param.z)/2.0;\n"
"  basis[4] = param.x*(1.0 + param.z)/2.0;\n"
"  basis[5] = param.y*(1.0 + param.z)/2.0;\n"
"}}\n"
"\n"
"void HGradWdgC1_basisGradientAt(in vec3 param, out float basisGradient[18]) // 6 * 3\n"
"{{\n"
"  basisGradient[ 0] = -(1.0 - param.z)/2.0;\n"
"  basisGradient[ 1] = -(1.0 - param.z)/2.0;\n"
"  basisGradient[ 2] = -(1.0 - param.x - param.y)/2.0;\n"
"\n"
"  basisGradient[ 3] =  (1.0 - param.z)/2.0;\n"
"  basisGradient[ 4] =   0.0;\n"
"  basisGradient[ 5] =  -param.x/2.0;\n"
"\n"
"  basisGradient[ 6] =   0.0;\n"
"  basisGradient[ 7] =  (1.0 - param.z)/2.0;\n"
"  basisGradient[ 8] =  -param.y/2.0;\n"
"\n"
"  basisGradient[ 9] = -(1.0 + param.z)/2.0;\n"
"  basisGradient[10] = -(1.0 + param.z)/2.0;\n"
"  basisGradient[11] =  (1.0 - param.x - param.y)/2.0;\n"
"\n"
"  basisGradient[12] =  (1.0 + param.z)/2.0;\n"
"  basisGradient[13] =   0.0;\n"
"  basisGradient[14] =   param.x/2.0;\n"
"\n"
"  basisGradient[15] =   0.0;\n"
"  basisGradient[16] =  (1.0 + param.z)/2.0;\n"
"  basisGradient[17] =   param.y/2.0;\n"
"}}\n"
"#endif /* BASIS_HGradWdgC1 */\n"
"\n"
"#ifdef SHAPE_wedge\n"
"int wedge_axisPermutationForSide(in int side)\n"
"{{\n"
"  switch(side)\n"
"  {{\n"
"  case 0: return 2; // (+r, +t) = -S\n"
"  case 1: return 6; // (-r+s, +t) = +RS\n"
"  case 2: return 0; // (-s, +t) = -R\n"
"  case 3: return 4; // (-r, -s) = -T\n"
"  case 4: return 5; // (+r, +s) = +T\n"
"  }}\n"
"}}\n"
"\n"
"vec3 wedge_normalToSideAt(in int sideId, in float shapeVals[{ShapeCoeffPerCell}], in vec3 rst, in vec3 cameraVector)\n"
"{{\n"
"  vec3 dxdr;\n"
"  vec3 dxds;\n"
"  vec3 dxdt;\n"
"  vec3 nn;\n"
"  shapeGradientAt(rst, shapeVals, dxdr, dxds, dxdt);\n"
"  switch (sideId)\n"
"  {{\n"
"  // Quadrilaterals\n"
"  case 0: nn = cross(dxdr, dxdt); break; // -S\n"
"  case 1: nn = cross(dxds - dxdr, dxdt); break; // +RS\n"
"  case 2: nn = cross(dxdt, dxds); break; // -R\n"
"  // Triangles\n"
"  case 3: nn = cross(dxds, dxdr); break; // -T\n"
"  case 4: nn = cross(dxdr, dxds); break; // +T\n"
"  // Bottom edges (in -T plane)\n"
"  case 5:\n"
"  case 6:\n"
"  case 7:\n"
"          nn = cross(dxds, dxdr); break; // Or cross(±dxd{{rs}}, dxdt);\n"
"  // Vertical (+T-directed) edges\n"
"  case 8: nn = -(dxdr + dxds); break;\n"
"  case 9: nn = dxdr - 0.5 * dxds; break;\n"
"  case 10: nn = dxds - 0.5 * dxdr; break;\n"
"  // Top edges (in +T plane)\n"
"  case 11: nn = cross(dxdr + dxds, dxdt); break; // Or cross(dxds - dxdr, dxdr + dxds);\n"
"  case 12: nn = cross(dxdt, dxds); break; // Or cross(dxds, dxdr);\n"
"  case 13: nn = cross(dxdt, dxds); break; // Or cross(dxds, dxdr);\n"
"  // Vertices (bottom, then top)\n"
"  case 14: nn = -(dxdr + dxds + 2*dxdt); break;\n"
"  case 15: nn = dxdr - 0.5 * dxds - 2 * dxdt; break;\n"
"  case 16: nn = dxds - 0.5 * dxdr - 2 * dxdt; break;\n"
"  case 17: nn = -(dxdr + dxds - 2*dxdt); break;\n"
"  case 18: nn = dxdr - 0.5 * dxds + 2 * dxdt; break;\n"
"  case 19: nn = dxds - 0.5 * dxdr + 2 * dxdt; break;\n"
"  default: nn = cameraVector; break;\n"
"  }}\n"
"  return normalize(nn);\n"
"}}\n"
"\n"
"void NoneWdgI0_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"void NoneWdgC1_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"#endif /* SHAPE_wedge */\n"
"\n"
"/** Hexahedral function space interpolation */\n"
"#if defined(BASIS_HGradHexI0) || defined(BASIS_HGradHexC0)\n"
"void HGradHexI0_basisAt(in vec3 param, out float basis[1])\n"
"{{\n"
"  basis[0] = 1.0;\n"
"}}\n"
"\n"
"void HGradHexI0_basisGradientAt(in vec3 param, out float basisGradient[3])\n"
"{{\n"
"  basisGradient[0] = 0.;\n"
"  basisGradient[1] = 0.;\n"
"  basisGradient[2] = 0.;\n"
"}}\n"
"\n"
"void HGradHexC0_basisAt(in vec3 param, out float basis[1])\n"
"{{\n"
"  HGradHexI0_basisAt(param, basis);\n"
"}}\n"
"\n"
"void HGradHexC0_basisGradientAt(in vec3 param, out float basisGradient[3])\n"
"{{\n"
"  HGradHexI0_basisGradientAt(param, basisGradient);\n"
"}}\n"
"#endif /* BASIS_HGradHexI0 */\n"
"\n"
"#ifdef BASIS_HGradHexC1\n"
"void HGradHexC1_basisAt(in vec3 param, out float basis[8])\n"
"{{\n"
"  basis[0] = (1.0 - param.x)*(1.0 - param.y)*(1.0 - param.z)/8.0;\n"
"  basis[1] = (1.0 + param.x)*(1.0 - param.y)*(1.0 - param.z)/8.0;\n"
"  basis[2] = (1.0 + param.x)*(1.0 + param.y)*(1.0 - param.z)/8.0;\n"
"  basis[3] = (1.0 - param.x)*(1.0 + param.y)*(1.0 - param.z)/8.0;\n"
"\n"
"  basis[4] = (1.0 - param.x)*(1.0 - param.y)*(1.0 + param.z)/8.0;\n"
"  basis[5] = (1.0 + param.x)*(1.0 - param.y)*(1.0 + param.z)/8.0;\n"
"  basis[6] = (1.0 + param.x)*(1.0 + param.y)*(1.0 + param.z)/8.0;\n"
"  basis[7] = (1.0 - param.x)*(1.0 + param.y)*(1.0 + param.z)/8.0;\n"
"}}\n"
"\n"
"void HGradHexC1_basisGradientAt(in vec3 param, out float basisGradient[24]) // 8 * 3\n"
"{{\n"
"  basisGradient[ 0] = -(1.0 - param.y)*(1.0 - param.z)/8.0;\n"
"  basisGradient[ 1] = -(1.0 - param.x)*(1.0 - param.z)/8.0;\n"
"  basisGradient[ 2] = -(1.0 - param.x)*(1.0 - param.y)/8.0;\n"
"\n"
"  basisGradient[ 3] =  (1.0 - param.y)*(1.0 - param.z)/8.0;\n"
"  basisGradient[ 4] = -(1.0 + param.x)*(1.0 - param.z)/8.0;\n"
"  basisGradient[ 5] = -(1.0 + param.x)*(1.0 - param.y)/8.0;\n"
"\n"
"  basisGradient[ 6] =  (1.0 + param.y)*(1.0 - param.z)/8.0;\n"
"  basisGradient[ 7] =  (1.0 + param.x)*(1.0 - param.z)/8.0;\n"
"  basisGradient[ 8] = -(1.0 + param.x)*(1.0 + param.y)/8.0;\n"
"\n"
"  basisGradient[ 9] = -(1.0 + param.y)*(1.0 - param.z)/8.0;\n"
"  basisGradient[10] =  (1.0 - param.x)*(1.0 - param.z)/8.0;\n"
"  basisGradient[11] = -(1.0 - param.x)*(1.0 + param.y)/8.0;\n"
"\n"
"  basisGradient[12] = -(1.0 - param.y)*(1.0 + param.z)/8.0;\n"
"  basisGradient[13] = -(1.0 - param.x)*(1.0 + param.z)/8.0;\n"
"  basisGradient[14] =  (1.0 - param.x)*(1.0 - param.y)/8.0;\n"
"\n"
"  basisGradient[15] =  (1.0 - param.y)*(1.0 + param.z)/8.0;\n"
"  basisGradient[16] = -(1.0 + param.x)*(1.0 + param.z)/8.0;\n"
"  basisGradient[17] =  (1.0 + param.x)*(1.0 - param.y)/8.0;\n"
"\n"
"  basisGradient[18] =  (1.0 + param.y)*(1.0 + param.z)/8.0;\n"
"  basisGradient[19] =  (1.0 + param.x)*(1.0 + param.z)/8.0;\n"
"  basisGradient[20] =  (1.0 + param.x)*(1.0 + param.y)/8.0;\n"
"\n"
"  basisGradient[21] = -(1.0 + param.y)*(1.0 + param.z)/8.0;\n"
"  basisGradient[22] =  (1.0 - param.x)*(1.0 + param.z)/8.0;\n"
"  basisGradient[23] =  (1.0 - param.x)*(1.0 + param.y)/8.0;\n"
"}}\n"
"#endif /* BASIS_HGradHexC1 */\n"
"\n"
"#ifdef BASIS_HGradHexC2\n"
"// NB: This is a complete 27-node hex.\n"
"//     Intrepid calls this a C2 vs an I2 hex since the full set\n"
"//     of tri-quadratic basis functions is included.\n"
"//     (C stands for Complete; I for Incomplete.)\n"
"void HGradHexC2_basisAt(in vec3 param, out float basis[27])\n"
"{{\n"
"  basis[ 0] = 0.125*(-1. + param.x)*param.x*(-1. + param.y)*param.y*(-1. + param.z)*param.z;\n"
"  basis[ 1] = 0.125*param.x*(1.+ param.x)*(-1. + param.y)*param.y*(-1. + param.z)*param.z;\n"
"  basis[ 2] = 0.125*param.x*(1.+ param.x)*param.y*(1.+ param.y)*(-1. + param.z)*param.z;\n"
"  basis[ 3] = 0.125*(-1. + param.x)*param.x*param.y*(1.+ param.y)*(-1. + param.z)*param.z;\n"
"  basis[ 4] = 0.125*(-1. + param.x)*param.x*(-1. + param.y)*param.y*param.z*(1.+ param.z);\n"
"  basis[ 5] = 0.125*param.x*(1.+ param.x)*(-1. + param.y)*param.y*param.z*(1.+ param.z);\n"
"  basis[ 6] = 0.125*param.x*(1.+ param.x)*param.y*(1.+ param.y)*param.z*(1.+ param.z);\n"
"  basis[ 7] = 0.125*(-1. + param.x)*param.x*param.y*(1.+ param.y)*param.z*(1.+ param.z);\n"
"\n"
"  basis[ 8] = 0.25*(1. - param.x)*(1. + param.x)*(-1. + param.y)*param.y*(-1. + param.z)*param.z;\n"
"  basis[ 9] = 0.25*param.x*(1.+ param.x)*(1. - param.y)*(1. + param.y)*(-1. + param.z)*param.z;\n"
"  basis[10] = 0.25*(1. - param.x)*(1. + param.x)*param.y*(1.+ param.y)*(-1. + param.z)*param.z;\n"
"  basis[11] = 0.25*(-1. + param.x)*param.x*(1. - param.y)*(1. + param.y)*(-1. + param.z)*param.z;\n"
"\n"
"  basis[12] = 0.25*(-1. + param.x)*param.x*(-1. + param.y)*param.y*(1. - param.z)*(1. + param.z);\n"
"  basis[13] = 0.25*param.x*(1.+ param.x)*(-1. + param.y)*param.y*(1. - param.z)*(1. + param.z);\n"
"  basis[14] = 0.25*param.x*(1.+ param.x)*param.y*(1.+ param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[15] = 0.25*(-1. + param.x)*param.x*param.y*(1.+ param.y)*(1. - param.z)*(1. + param.z);\n"
"\n"
"  basis[16] = 0.25*(1. - param.x)*(1. + param.x)*(-1. + param.y)*param.y*param.z*(1.+ param.z);\n"
"  basis[17] = 0.25*param.x*(1.+ param.x)*(1. - param.y)*(1. + param.y)*param.z*(1.+ param.z);\n"
"  basis[18] = 0.25*(1. - param.x)*(1. + param.x)*param.y*(1.+ param.y)*param.z*(1.+ param.z);\n"
"  basis[19] = 0.25*(-1. + param.x)*param.x*(1. - param.y)*(1. + param.y)*param.z*(1.+ param.z);\n"
"\n"
"  basis[20] = (1. - param.x)*(1. + param.x)*(1. - param.y)*(1. + param.y)*(1. - param.z)*(1. + param.z);\n"
"\n"
"  basis[21] = 0.5*(1. - param.x)*(1. + param.x)*(1. - param.y)*(1. + param.y)*(-1. + param.z)*param.z;\n"
"  basis[22] = 0.5*(1. - param.x)*(1. + param.x)*(1. - param.y)*(1. + param.y)*param.z*(1.+ param.z);\n"
"  basis[23] = 0.5*(-1. + param.x)*param.x*(1. - param.y)*(1. + param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[24] = 0.5*param.x*(1.+ param.x)*(1. - param.y)*(1. + param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[25] = 0.5*(1. - param.x)*(1. + param.x)*(-1. + param.y)*param.y*(1. - param.z)*(1. + param.z);\n"
"  basis[26] = 0.5*(1. - param.x)*(1. + param.x)*param.y*(1.+ param.y)*(1. - param.z)*(1. + param.z);\n"
"}}\n"
"\n"
"void HGradHexC2_basisGradientAt(in vec3 param, out float basisGradient[81]) // 27 * 3\n"
"{{\n"
"  basis[ 0 * 3 + 0] = (-0.125 + 0.25*param.x)*(-1. + param.y)*param.y*(-1. + param.z)*param.z;\n"
"  basis[ 0 * 3 + 1] = (-1. + param.x)*param.x*(-0.125 + 0.25*param.y)*(-1. + param.z)*param.z;\n"
"  basis[ 0 * 3 + 2] = (-1. + param.x)*param.x*(-1. + param.y)*param.y*(-0.125 + 0.25*param.z);\n"
"\n"
"  basis[ 1 * 3 + 0] = (0.125 + 0.25*param.x)*(-1. + param.y)*param.y*(-1. + param.z)*param.z;\n"
"  basis[ 1 * 3 + 1] = param.x*(1. + param.x)*(-0.125 + 0.25*param.y)*(-1. + param.z)*param.z;\n"
"  basis[ 1 * 3 + 2] = param.x*(1. + param.x)*(-1. + param.y)*param.y*(-0.125 + 0.25*param.z);\n"
"\n"
"  basis[ 2 * 3 + 0] = (0.125 + 0.25*param.x)*param.y*(1. + param.y)*(-1. + param.z)*param.z;\n"
"  basis[ 2 * 3 + 1] = param.x*(1. + param.x)*(0.125 + 0.25*param.y)*(-1. + param.z)*param.z;\n"
"  basis[ 2 * 3 + 2] = param.x*(1. + param.x)*param.y*(1. + param.y)*(-0.125 + 0.25*param.z);\n"
"\n"
"  basis[ 3 * 3 + 0] = (-0.125 + 0.25*param.x)*param.y*(1. + param.y)*(-1. + param.z)*param.z;\n"
"  basis[ 3 * 3 + 1] = (-1. + param.x)*param.x*(0.125 + 0.25*param.y)*(-1. + param.z)*param.z;\n"
"  basis[ 3 * 3 + 2] = (-1. + param.x)*param.x*param.y*(1. + param.y)*(-0.125 + 0.25*param.z);\n"
"\n"
"  basis[ 4 * 3 + 0] = (-0.125 + 0.25*param.x)*(-1. + param.y)*param.y*param.z*(1. + param.z);\n"
"  basis[ 4 * 3 + 1] = (-1. + param.x)*param.x*(-0.125 + 0.25*param.y)*param.z*(1. + param.z);\n"
"  basis[ 4 * 3 + 2] = (-1. + param.x)*param.x*(-1. + param.y)*param.y*(0.125 + 0.25*param.z);\n"
"\n"
"  basis[ 5 * 3 + 0] = (0.125 + 0.25*param.x)*(-1. + param.y)*param.y*param.z*(1. + param.z);\n"
"  basis[ 5 * 3 + 1] = param.x*(1. + param.x)*(-0.125 + 0.25*param.y)*param.z*(1. + param.z);\n"
"  basis[ 5 * 3 + 2] = param.x*(1. + param.x)*(-1. + param.y)*param.y*(0.125 + 0.25*param.z);\n"
"\n"
"  basis[ 6 * 3 + 0] = (0.125 + 0.25*param.x)*param.y*(1. + param.y)*param.z*(1. + param.z);\n"
"  basis[ 6 * 3 + 1] = param.x*(1. + param.x)*(0.125 + 0.25*param.y)*param.z*(1. + param.z);\n"
"  basis[ 6 * 3 + 2] = param.x*(1. + param.x)*param.y*(1. + param.y)*(0.125 + 0.25*param.z);\n"
"\n"
"  basis[ 7 * 3 + 0] = (-0.125 + 0.25*param.x)*param.y*(1. + param.y)*param.z*(1. + param.z);\n"
"  basis[ 7 * 3 + 1] = (-1. + param.x)*param.x*(0.125 + 0.25*param.y)*param.z*(1. + param.z);\n"
"  basis[ 7 * 3 + 2] = (-1. + param.x)*param.x*param.y*(1. + param.y)*(0.125 + 0.25*param.z);\n"
"\n"
"  basis[ 8 * 3 + 0] = -0.5*param.x*(-1. + param.y)*param.y*(-1. + param.z)*param.z;\n"
"  basis[ 8 * 3 + 1] = (1. - param.x)*(1. + param.x)*(-0.25 + 0.5*param.y)*(-1. + param.z)*param.z;\n"
"  basis[ 8 * 3 + 2] = (1. - param.x)*(1. + param.x)*(-1. + param.y)*param.y*(-0.25 + 0.5*param.z);\n"
"\n"
"  basis[ 9 * 3 + 0] = (0.25 + 0.5*param.x)*(1. - param.y)*(1. + param.y)*(-1. + param.z)*param.z;\n"
"  basis[ 9 * 3 + 1] = param.x*(1. + param.x)*(-0.5*param.y)*(-1. + param.z)*param.z;\n"
"  basis[ 9 * 3 + 2] = param.x*(1. + param.x)*(1. - param.y)*(1. + param.y)*(-0.25 + 0.5*param.z);\n"
"\n"
"  basis[10 * 3 + 0] = -0.5*param.x*param.y*(1. + param.y)*(-1. + param.z)*param.z;\n"
"  basis[10 * 3 + 1] = (1. - param.x)*(1. + param.x)*(0.25 + 0.5*param.y)*(-1. + param.z)*param.z;\n"
"  basis[10 * 3 + 2] = (1. - param.x)*(1. + param.x)*param.y*(1. + param.y)*(-0.25 + 0.5*param.z);\n"
"\n"
"  basis[11 * 3 + 0] = (-0.25 + 0.5*param.x)*(1. - param.y)*(1. + param.y)*(-1. + param.z)*param.z;\n"
"  basis[11 * 3 + 1] = (-1. + param.x)*param.x*(-0.5*param.y)*(-1. + param.z)*param.z;\n"
"  basis[11 * 3 + 2] = (-1. + param.x)*param.x*(1. - param.y)*(1. + param.y)*(-0.25 + 0.5*param.z);\n"
"\n"
"  basis[12 * 3 + 0] = (-0.25 + 0.5*param.x)*(-1. + param.y)*param.y*(1. - param.z)*(1. + param.z);\n"
"  basis[12 * 3 + 1] = (-1. + param.x)*param.x*(-0.25 + 0.5*param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[12 * 3 + 2] = (-1. + param.x)*param.x*(-1. + param.y)*param.y*(-0.5*param.z);\n"
"\n"
"  basis[13 * 3 + 0] = (0.25 + 0.5*param.x)*(-1. + param.y)*param.y*(1. - param.z)*(1. + param.z);\n"
"  basis[13 * 3 + 1] = param.x*(1. + param.x)*(-0.25 + 0.5*param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[13 * 3 + 2] = param.x*(1. + param.x)*(-1. + param.y)*param.y*(-0.5*param.z);\n"
"\n"
"  basis[14 * 3 + 0] = (0.25 + 0.5*param.x)*param.y*(1. + param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[14 * 3 + 1] = param.x*(1. + param.x)*(0.25 + 0.5*param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[14 * 3 + 2] = param.x*(1. + param.x)*param.y*(1. + param.y)*(-0.5*param.z);\n"
"\n"
"  basis[15 * 3 + 0] = (-0.25 + 0.5*param.x)*param.y*(1. + param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[15 * 3 + 1] = (-1. + param.x)*param.x*(0.25 + 0.5*param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[15 * 3 + 2] = (-1. + param.x)*param.x*param.y*(1. + param.y)*(-0.5*param.z);\n"
"\n"
"  basis[16 * 3 + 0] = -0.5*param.x*(-1. + param.y)*param.y*param.z*(1. + param.z);\n"
"  basis[16 * 3 + 1] = (1. - param.x)*(1. + param.x)*(-0.25 + 0.5*param.y)*param.z*(1. + param.z);\n"
"  basis[16 * 3 + 2] = (1. - param.x)*(1. + param.x)*(-1. + param.y)*param.y*(0.25 + 0.5*param.z);\n"
"\n"
"  basis[17 * 3 + 0] = (0.25 + 0.5*param.x)*(1. - param.y)*(1. + param.y)*param.z*(1. + param.z);\n"
"  basis[17 * 3 + 1] = param.x*(1. + param.x)*(-0.5*param.y)*param.z*(1. + param.z);\n"
"  basis[17 * 3 + 2] = param.x*(1. + param.x)*(1. - param.y)*(1. + param.y)*(0.25 + 0.5*param.z);\n"
"\n"
"  basis[18 * 3 + 0] = -0.5*param.x*param.y*(1. + param.y)*param.z*(1. + param.z);\n"
"  basis[18 * 3 + 1] = (1. - param.x)*(1. + param.x)*(0.25 + 0.5*param.y)*param.z*(1. + param.z);\n"
"  basis[18 * 3 + 2] = (1. - param.x)*(1. + param.x)*param.y*(1. + param.y)*(0.25 + 0.5*param.z);\n"
"\n"
"  basis[19 * 3 + 0] = (-0.25 + 0.5*param.x)*(1. - param.y)*(1. + param.y)*param.z*(1. + param.z);\n"
"  basis[19 * 3 + 1] = (-1. + param.x)*param.x*(-0.5*param.y)*param.z*(1. + param.z);\n"
"  basis[19 * 3 + 2] = (-1. + param.x)*param.x*(1. - param.y)*(1. + param.y)*(0.25 + 0.5*param.z);\n"
"\n"
"  basis[20 * 3 + 0] = -2.*param.x*(1. - param.y)*(1. + param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[20 * 3 + 1] = (1. - param.x)*(1. + param.x)*(-2.*param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[20 * 3 + 2] = (1. - param.x)*(1. + param.x)*(1. - param.y)*(1. + param.y)*(-2.*param.z);\n"
"\n"
"  basis[21 * 3 + 0] = -param.x*(1. - param.y)*(1. + param.y)*(-1. + param.z)*param.z;\n"
"  basis[21 * 3 + 1] = (1. - param.x)*(1. + param.x)*(-param.y)*(-1. + param.z)*param.z;\n"
"  basis[21 * 3 + 2] = (1. - param.x)*(1. + param.x)*(1. - param.y)*(1. + param.y)*(-0.5 + param.z);\n"
"\n"
"  basis[22 * 3 + 0] = -param.x*(1. - param.y)*(1. + param.y)*param.z*(1. + param.z);\n"
"  basis[22 * 3 + 1] = (1. - param.x)*(1. + param.x)*(-param.y)*param.z*(1. + param.z);\n"
"  basis[22 * 3 + 2] = (1. - param.x)*(1. + param.x)*(1. - param.y)*(1. + param.y)*(0.5 + param.z);\n"
"\n"
"  basis[23 * 3 + 0] = (-0.5 + param.x)*(1. - param.y)*(1. + param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[23 * 3 + 1] = (-1. + param.x)*param.x*(-param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[23 * 3 + 2] = (-1. + param.x)*param.x*(1. - param.y)*(1. + param.y)*(-param.z);\n"
"\n"
"  basis[24 * 3 + 0] = (0.5 + param.x)*(1. - param.y)*(1. + param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[24 * 3 + 1] = param.x*(1. + param.x)*(-param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[24 * 3 + 2] = param.x*(1. + param.x)*(1. - param.y)*(1. + param.y)*(-param.z);\n"
"\n"
"  basis[25 * 3 + 0] = -param.x*(-1. + param.y)*param.y*(1. - param.z)*(1. + param.z);\n"
"  basis[25 * 3 + 1] = (1. - param.x)*(1. + param.x)*(-0.5 + param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[25 * 3 + 2] = (1. - param.x)*(1. + param.x)*(-1. + param.y)*param.y*(-param.z);\n"
"\n"
"  basis[26 * 3 + 0] = -param.x*param.y*(1. + param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[26 * 3 + 1] = (1. - param.x)*(1. + param.x)*(0.5 + param.y)*(1. - param.z)*(1. + param.z);\n"
"  basis[26 * 3 + 2] = (1. - param.x)*(1. + param.x)*param.y*(1. + param.y)*(-param.z);\n"
"}}\n"
"#endif /* BASIS_HGradHexC2 */\n"
"\n"
"#ifdef BASIS_HGradHexI2\n"
"// NB: This is a 20-node hex (mid-edge but not mid-face or mid-body points).\n"
"//     Intrepid calls this an I2 vs a C2 hex since it omits mid-face/body points.\n"
"void HGradHexI2_basisAt(in vec3 param, out float basis[20])\n"
"{{\n"
"  basis[ 0] = 0.125*(1.0 - param.x)*(1.0 - param.y)*(1.0 - param.z)*(-param.x - param.y - param.z - 2.0);\n"
"  basis[ 1] = 0.125*(1.0 + param.x)*(1.0 - param.y)*(1.0 - param.z)*( param.x - param.y - param.z - 2.0);\n"
"  basis[ 2] = 0.125*(1.0 + param.x)*(1.0 + param.y)*(1.0 - param.z)*( param.x + param.y - param.z - 2.0);\n"
"  basis[ 3] = 0.125*(1.0 - param.x)*(1.0 + param.y)*(1.0 - param.z)*(-param.x + param.y - param.z - 2.0);\n"
"  basis[ 4] = 0.125*(1.0 - param.x)*(1.0 - param.y)*(1.0 + param.z)*(-param.x - param.y + param.z - 2.0);\n"
"  basis[ 5] = 0.125*(1.0 + param.x)*(1.0 - param.y)*(1.0 + param.z)*( param.x - param.y + param.z - 2.0);\n"
"  basis[ 6] = 0.125*(1.0 + param.x)*(1.0 + param.y)*(1.0 + param.z)*( param.x + param.y + param.z - 2.0);\n"
"  basis[ 7] = 0.125*(1.0 - param.x)*(1.0 + param.y)*(1.0 + param.z)*(-param.x + param.y + param.z - 2.0);\n"
"\n"
"  basis[ 8] = 0.25*(1.0 - param.x*param.x)*(1.0 - param.y)*(1.0 - param.z);\n"
"  basis[ 9] = 0.25*(1.0 + param.x)*(1.0 - param.y*param.y)*(1.0 - param.z);\n"
"  basis[10] = 0.25*(1.0 - param.x*param.x)*(1.0 + param.y)*(1.0 - param.z);\n"
"  basis[11] = 0.25*(1.0 - param.x)*(1.0 - param.y*param.y)*(1.0 - param.z);\n"
"\n"
"  basis[12] = 0.25*(1.0 - param.x)*(1.0 - param.y)*(1.0 - param.z*param.z);\n"
"  basis[13] = 0.25*(1.0 + param.x)*(1.0 - param.y)*(1.0 - param.z*param.z);\n"
"  basis[14] = 0.25*(1.0 + param.x)*(1.0 + param.y)*(1.0 - param.z*param.z);\n"
"  basis[15] = 0.25*(1.0 - param.x)*(1.0 + param.y)*(1.0 - param.z*param.z);\n"
"\n"
"  basis[16] = 0.25*(1.0 - param.x*param.x)*(1.0 - param.y)*(1.0 + param.z);\n"
"  basis[17] = 0.25*(1.0 + param.x)*(1.0 - param.y*param.y)*(1.0 + param.z);\n"
"  basis[18] = 0.25*(1.0 - param.x*param.x)*(1.0 + param.y)*(1.0 + param.z);\n"
"  basis[19] = 0.25*(1.0 - param.x)*(1.0 - param.y*param.y)*(1.0 + param.z);\n"
"}}\n"
"\n"
"void HGradHexI2_basisGradientAt(in vec3 param, out float basisGradient[60]) // 20 * 3\n"
"{{\n"
"  basisGradient[0 * 3 + 0] = -0.125*(1.0-param.y)*(1.0-param.z)*(-param.x-param.y-param.z-2.0) - 0.125*(1.0-param.x)*(1.0-param.y)*(1.0-param.z);\n"
"  basisGradient[0 * 3 + 1] = -0.125*(1.0-param.x)*(1.0-param.z)*(-param.x-param.y-param.z-2.0) - 0.125*(1.0-param.x)*(1.0-param.y)*(1.0-param.z);\n"
"  basisGradient[0 * 3 + 2] = -0.125*(1.0-param.x)*(1.0-param.y)*(-param.x-param.y-param.z-2.0) - 0.125*(1.0-param.x)*(1.0-param.y)*(1.0-param.z);\n"
"\n"
"  basisGradient[1 * 3 + 0] =  0.125*(1.0-param.y)*(1.0-param.z)*( param.x-param.y-param.z-2.0) + 0.125*(1.0+param.x)*(1.0-param.y)*(1.0-param.z);\n"
"  basisGradient[1 * 3 + 1] = -0.125*(1.0+param.x)*(1.0-param.z)*( param.x-param.y-param.z-2.0) - 0.125*(1.0+param.x)*(1.0-param.y)*(1.0-param.z);\n"
"  basisGradient[1 * 3 + 2] = -0.125*(1.0+param.x)*(1.0-param.y)*( param.x-param.y-param.z-2.0) - 0.125*(1.0+param.x)*(1.0-param.y)*(1.0-param.z);\n"
"\n"
"  basisGradient[2 * 3 + 0] =  0.125*(1.0+param.y)*(1.0-param.z)*( param.x+param.y-param.z-2.0) + 0.125*(1.0+param.x)*(1.0+param.y)*(1.0-param.z);\n"
"  basisGradient[2 * 3 + 1] =  0.125*(1.0+param.x)*(1.0-param.z)*( param.x+param.y-param.z-2.0) + 0.125*(1.0+param.x)*(1.0+param.y)*(1.0-param.z);\n"
"  basisGradient[2 * 3 + 2] = -0.125*(1.0+param.x)*(1.0+param.y)*( param.x+param.y-param.z-2.0) - 0.125*(1.0+param.x)*(1.0+param.y)*(1.0-param.z);\n"
"\n"
"  basisGradient[3 * 3 + 0] = -0.125*(1.0+param.y)*(1.0-param.z)*(-param.x+param.y-param.z-2.0) - 0.125*(1.0-param.x)*(1.0+param.y)*(1.0-param.z);\n"
"  basisGradient[3 * 3 + 1] =  0.125*(1.0-param.x)*(1.0-param.z)*(-param.x+param.y-param.z-2.0) + 0.125*(1.0-param.x)*(1.0+param.y)*(1.0-param.z);\n"
"  basisGradient[3 * 3 + 2] = -0.125*(1.0-param.x)*(1.0+param.y)*(-param.x+param.y-param.z-2.0) - 0.125*(1.0-param.x)*(1.0+param.y)*(1.0-param.z);\n"
"\n"
"  basisGradient[4 * 3 + 0] = -0.125*(1.0-param.y)*(1.0+param.z)*(-param.x-param.y+param.z-2.0) - 0.125*(1.0-param.x)*(1.0-param.y)*(1.0+param.z);\n"
"  basisGradient[4 * 3 + 1] = -0.125*(1.0-param.x)*(1.0+param.z)*(-param.x-param.y+param.z-2.0) - 0.125*(1.0-param.x)*(1.0-param.y)*(1.0+param.z);\n"
"  basisGradient[4 * 3 + 2] =  0.125*(1.0-param.x)*(1.0-param.y)*(-param.x-param.y+param.z-2.0) + 0.125*(1.0-param.x)*(1.0-param.y)*(1.0+param.z);\n"
"\n"
"  basisGradient[5 * 3 + 0] =  0.125*(1.0-param.y)*(1.0+param.z)*( param.x-param.y+param.z-2.0) + 0.125*(1.0+param.x)*(1.0-param.y)*(1.0+param.z);\n"
"  basisGradient[5 * 3 + 1] = -0.125*(1.0+param.x)*(1.0+param.z)*( param.x-param.y+param.z-2.0) - 0.125*(1.0+param.x)*(1.0-param.y)*(1.0+param.z);\n"
"  basisGradient[5 * 3 + 2] =  0.125*(1.0+param.x)*(1.0-param.y)*( param.x-param.y+param.z-2.0) + 0.125*(1.0+param.x)*(1.0-param.y)*(1.0+param.z);\n"
"\n"
"  basisGradient[6 * 3 + 0] =  0.125*(1.0+param.y)*(1.0+param.z)*( param.x+param.y+param.z-2.0) + 0.125*(1.0+param.x)*(1.0+param.y)*(1.0+param.z);\n"
"  basisGradient[6 * 3 + 1] =  0.125*(1.0+param.x)*(1.0+param.z)*( param.x+param.y+param.z-2.0) + 0.125*(1.0+param.x)*(1.0+param.y)*(1.0+param.z);\n"
"  basisGradient[6 * 3 + 2] =  0.125*(1.0+param.x)*(1.0+param.y)*( param.x+param.y+param.z-2.0) + 0.125*(1.0+param.x)*(1.0+param.y)*(1.0+param.z);\n"
"\n"
"  basisGradient[7 * 3 + 0] = -0.125*(1.0+param.y)*(1.0+param.z)*(-param.x+param.y+param.z-2.0) - 0.125*(1.0-param.x)*(1.0+param.y)*(1.0+param.z);\n"
"  basisGradient[7 * 3 + 1] =  0.125*(1.0-param.x)*(1.0+param.z)*(-param.x+param.y+param.z-2.0) + 0.125*(1.0-param.x)*(1.0+param.y)*(1.0+param.z);\n"
"  basisGradient[7 * 3 + 2] =  0.125*(1.0-param.x)*(1.0+param.y)*(-param.x+param.y+param.z-2.0) + 0.125*(1.0-param.x)*(1.0+param.y)*(1.0+param.z);\n"
"\n"
"  basisGradient[8 * 3 + 0] = -0.5*param.x*(1.0-param.y)*(1.0-param.z);\n"
"  basisGradient[8 * 3 + 1] = -0.25*(1.0-param.x*param.x)*(1.0-param.z);\n"
"  basisGradient[8 * 3 + 2] = -0.25*(1.0-param.x*param.x)*(1.0-param.y);\n"
"\n"
"  basisGradient[9 * 3 + 0] =  0.25*(1.0-param.y*param.y)*(1.0-param.z);\n"
"  basisGradient[9 * 3 + 1] = -0.5*param.y*(1.0+param.x)*(1.0-param.z);\n"
"  basisGradient[9 * 3 + 2] = -0.25*(1.0+param.x)*(1.0-param.y*param.y);\n"
"\n"
"  basisGradient[10 * 3 + 0] = -0.5*param.x*(1.0+param.y)*(1.0-param.z);\n"
"  basisGradient[10 * 3 + 1] =  0.25*(1.0-param.x*param.x)*(1.0-param.z);\n"
"  basisGradient[10 * 3 + 2] = -0.25*(1.0-param.x*param.x)*(1.0+param.y);\n"
"\n"
"  basisGradient[11 * 3 + 0] = -0.25*(1.0-param.y*param.y)*(1.0-param.z);\n"
"  basisGradient[11 * 3 + 1] = -0.5*param.y*(1.0-param.x)*(1.0-param.z);\n"
"  basisGradient[11 * 3 + 2] = -0.25*(1.0-param.x)*(1.0-param.y*param.y);\n"
"\n"
"  basisGradient[12 * 3 + 0] = -0.25*(1.0-param.y)*(1.0-param.z*param.z);\n"
"  basisGradient[12 * 3 + 1] = -0.25*(1.0-param.x)*(1.0-param.z*param.z);\n"
"  basisGradient[12 * 3 + 2] = -0.5*param.z*(1.0-param.x)*(1.0-param.y);\n"
"\n"
"  basisGradient[13 * 3 + 0] =  0.25*(1.0-param.y)*(1.0-param.z*param.z);\n"
"  basisGradient[13 * 3 + 1] = -0.25*(1.0+param.x)*(1.0-param.z*param.z);\n"
"  basisGradient[13 * 3 + 2] = -0.5*param.z*(1.0+param.x)*(1.0-param.y);\n"
"\n"
"  basisGradient[14 * 3 + 0] =  0.25*(1.0+param.y)*(1.0-param.z*param.z);\n"
"  basisGradient[14 * 3 + 1] =  0.25*(1.0+param.x)*(1.0-param.z*param.z);\n"
"  basisGradient[14 * 3 + 2] = -0.5*param.z*(1.0+param.x)*(1.0+param.y);\n"
"\n"
"  basisGradient[15 * 3 + 0] = -0.25*(1.0+param.y)*(1.0-param.z*param.z);\n"
"  basisGradient[15 * 3 + 1] =  0.25*(1.0-param.x)*(1.0-param.z*param.z);\n"
"  basisGradient[15 * 3 + 2] = -0.5*param.z*(1.0-param.x)*(1.0+param.y);\n"
"\n"
"  basisGradient[16 * 3 + 0] = -0.5*param.x*(1.0-param.y)*(1.0+param.z);\n"
"  basisGradient[16 * 3 + 1] = -0.25*(1.0-param.x*param.x)*(1.0+param.z);\n"
"  basisGradient[16 * 3 + 2] =  0.25*(1.0-param.x*param.x)*(1.0-param.y);\n"
"\n"
"  basisGradient[17 * 3 + 0] =  0.25*(1.0-param.y*param.y)*(1.0+param.z);\n"
"  basisGradient[17 * 3 + 1] = -0.5*param.y*(1.0+param.x)*(1.0+param.z);\n"
"  basisGradient[17 * 3 + 2] =  0.25*(1.0+param.x)*(1.0-param.y*param.y);\n"
"\n"
"  basisGradient[18 * 3 + 0] = -0.5*param.x*(1.0+param.y)*(1.0+param.z);\n"
"  basisGradient[18 * 3 + 1] =  0.25*(1.0-param.x*param.x)*(1.0+param.z);\n"
"  basisGradient[18 * 3 + 2] =  0.25*(1.0-param.x*param.x)*(1.0+param.y);\n"
"\n"
"  basisGradient[19 * 3 + 0] = -0.25*(1.0-param.y*param.y)*(1.0+param.z);\n"
"  basisGradient[19 * 3 + 1] = -0.5*param.y*(1.0-param.x)*(1.0+param.z);\n"
"  basisGradient[19 * 3 + 2] =  0.25*(1.0-param.x)*(1.0-param.y*param.y);\n"
"}}\n"
"#endif /* BASIS_HGradHexI2 */\n"
"\n"
"#ifdef BASIS_HCurlHexI1\n"
"void HCurlHexI1_basisAt(in vec3 param, out float basis[36]) // 12 * 3\n"
"{{\n"
"  basis[0 * 3 + 0] = (1.0 - param.y)*(1.0 - param.z)/8.0;\n"
"  basis[0 * 3 + 1] = 0.0;\n"
"  basis[0 * 3 + 2] = 0.0;\n"
"\n"
"  basis[1 * 3 + 0] = 0.0;\n"
"  basis[1 * 3 + 1] = (1.0 + param.x)*(1.0 - param.z)/8.0;\n"
"  basis[1 * 3 + 2] = 0.0;\n"
"\n"
"  basis[2 * 3 + 0] = -(1.0 + param.y)*(1.0 - param.z)/8.0;\n"
"  basis[2 * 3 + 1] = 0.0;\n"
"  basis[2 * 3 + 2] = 0.0;\n"
"\n"
"  basis[3 * 3 + 0] = 0.0;\n"
"  basis[3 * 3 + 1] = -(1.0 - param.x)*(1.0 - param.z)/8.0;\n"
"  basis[3 * 3 + 2] = 0.0;\n"
"\n"
"  basis[4 * 3 + 0] = (1.0 - param.y)*(1.0 + param.z)/8.0;\n"
"  basis[4 * 3 + 1] = 0.0;\n"
"  basis[4 * 3 + 2] = 0.0;\n"
"\n"
"  basis[5 * 3 + 0] = 0.0;\n"
"  basis[5 * 3 + 1] = (1.0 + param.x)*(1.0 + param.z)/8.0;\n"
"  basis[5 * 3 + 2] = 0.0;\n"
"\n"
"  basis[6 * 3 + 0] = -(1.0 + param.y)*(1.0 + param.z)/8.0;\n"
"  basis[6 * 3 + 1] = 0.0;\n"
"  basis[6 * 3 + 2] = 0.0;\n"
"\n"
"  basis[7 * 3 + 0] = 0.0;\n"
"  basis[7 * 3 + 1] = -(1.0 - param.x)*(1.0 + param.z)/8.0;\n"
"  basis[7 * 3 + 2] = 0.0;\n"
"\n"
"  basis[8 * 3 + 0] = 0.0;\n"
"  basis[8 * 3 + 1] = 0.0;\n"
"  basis[8 * 3 + 2] = (1.0 - param.x)*(1.0 - param.y)/8.0;\n"
"\n"
"  basis[9 * 3 + 0] = 0.0;\n"
"  basis[9 * 3 + 1] = 0.0;\n"
"  basis[9 * 3 + 2] = (1.0 + param.x)*(1.0 - param.y)/8.0;\n"
"\n"
"  basis[10 * 3 + 0] = 0.0;\n"
"  basis[10 * 3 + 1] = 0.0;\n"
"  basis[10 * 3 + 2] = (1.0 + param.x)*(1.0 + param.y)/8.0;\n"
"\n"
"  basis[11 * 3 + 0] = 0.0;\n"
"  basis[11 * 3 + 1] = 0.0;\n"
"  basis[11 * 3 + 2] = (1.0 - param.x)*(1.0 + param.y)/8.0;\n"
"}}\n"
"#endif /* BASIS_HCurlHexI1 */\n"
"\n"
"#ifdef BASIS_HDivHexI1\n"
"void HDivHexI1_basisAt(in vec3 param, out float basis[18]) // 6 * 3\n"
"{{\n"
"  basis[0 * 3 + 0] = 0.0;\n"
"  basis[0 * 3 + 1] = (param.y - 1.0)/8.0;\n"
"  basis[0 * 3 + 2] = 0.0;\n"
"\n"
"  basis[1 * 3 + 0] = (1.0 + param.x)/8.0;\n"
"  basis[1 * 3 + 1] = 0.0;\n"
"  basis[1 * 3 + 2] = 0.0;\n"
"\n"
"  basis[2 * 3 + 0] = 0.0;\n"
"  basis[2 * 3 + 1] = (1.0 + param.y)/8.0;\n"
"  basis[2 * 3 + 2] = 0.0;\n"
"\n"
"  basis[3 * 3 + 0] = (param.x - 1.0)/8.0;\n"
"  basis[3 * 3 + 1] = 0.0;\n"
"  basis[3 * 3 + 2] = 0.0;\n"
"\n"
"  basis[4 * 3 + 0] = 0.0;\n"
"  basis[4 * 3 + 1] = 0.0;\n"
"  basis[4 * 3 + 2] = (param.z - 1.0)/8.0;\n"
"\n"
"  basis[5 * 3 + 0] = 0.0;\n"
"  basis[5 * 3 + 1] = 0.0;\n"
"  basis[5 * 3 + 2] = (1.0 + param.z)/8.0;\n"
"}}\n"
"#endif /* BASIS_HDivHexI1 */\n"
"\n"
"#ifdef SHAPE_hexahedron\n"
"int hexahedron_axisPermutationForSide(in int side)\n"
"{{\n"
"  switch(side)\n"
"  {{\n"
"  case 0: return 2; // (+r, +t) = -S\n"
"  case 1: return 1; // (+s, +t) = +R\n"
"  case 2: return 3; // (-r, +t) = +S\n"
"  case 3: return 0; // (-s, +t) = -R\n"
"  case 4: return 4; // (-r, -s) = -T\n"
"  case 5: return 5; // (+r, +s) = +T\n"
"  // case 1: return 6; // (-r+s,-r-s+t) = +RST\n"
"  // case 2: return 0; // (-s, +t) = -R\n"
"  // case 3: return 4; // (-r, -s) = -T\n"
"  }}\n"
"}}\n"
"\n"
"vec3 hexahedron_normalToSideAt(in int sideId, in float shapeVals[{ShapeCoeffPerCell}], in vec3 rst, in vec3 cameraVector)\n"
"{{\n"
"  vec3 dxdr;\n"
"  vec3 dxds;\n"
"  vec3 dxdt;\n"
"  vec3 nn;\n"
"  shapeGradientAt(rst, shapeVals, dxdr, dxds, dxdt);\n"
"  switch (sideId)\n"
"  {{\n"
"  // Quadrilaterals\n"
"  case  0: nn = cross(dxdr, dxdt); break; // -S\n"
"  case  1: nn = cross(dxds, dxdt); break; // +R\n"
"  case  2: nn = cross(dxdt, dxdr); break; // +S\n"
"  case  3: nn = cross(dxdt, dxds); break; // -R\n"
"  case  4: nn = cross(dxds, dxdr); break; // -T\n"
"  case  5: nn = cross(dxdr, dxds); break; // +T\n"
"  // Bottom edges (in -T plane)\n"
"  case  6: nn = -dxds - dxdt; break;\n"
"  case  7: nn =  dxdr - dxdt; break;\n"
"  case  8: nn =  dxds - dxdt; break;\n"
"  case  9: nn = -dxdr - dxdt; break;\n"
"  // Vertical (+T-directed) edges\n"
"  case 10: nn = -dxdr - dxds; break;\n"
"  case 11: nn =  dxdr - dxds; break;\n"
"  case 12: nn =  dxdr + dxds; break;\n"
"  case 13: nn = -dxdr + dxds; break;\n"
"  // Top edges (in +T plane)\n"
"  case 14: nn = -dxds + dxdt; break;\n"
"  case 15: nn =  dxdr + dxdt; break;\n"
"  case 16: nn =  dxds + dxdt; break;\n"
"  case 17: nn = -dxdr + dxdt; break;\n"
"  // Vertices (bottom, then top)\n"
"  case 18: nn = -dxdr - dxds - dxdt; break;\n"
"  case 19: nn =  dxdr - dxds - dxdt; break;\n"
"  case 20: nn =  dxdr + dxds - dxdt; break;\n"
"  case 21: nn = -dxdr + dxds - dxdt; break;\n"
"  case 22: nn = -dxdr - dxds + dxdt; break;\n"
"  case 23: nn =  dxdr - dxds + dxdt; break;\n"
"  case 24: nn =  dxdr + dxds + dxdt; break;\n"
"  case 25: nn = -dxdr + dxds + dxdt; break;\n"
"  default: nn = cameraVector; break;\n"
"  }}\n"
"  return normalize(nn);\n"
"}}\n"
"\n"
"void NoneHexI0_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"void NoneHexC1_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"#endif /* SHAPE_hexahedron */\n"
"\n"
"/** Tetrahedral function space interpolation */\n"
"#if defined(BASIS_HGradTetI0) || defined(BASIS_HGradTetC0)\n"
"void HGradTetC0_basisAt(in vec3 param, out float basis[1])\n"
"{{\n"
"  basis[0] = 1.0;\n"
"}}\n"
"\n"
"void HGradTetC0_basisGradientAt(in vec3 xx, out float basisGradient[3])\n"
"{{\n"
"  basisGradient[0] = 0.;\n"
"  basisGradient[1] = 0.;\n"
"  basisGradient[2] = 0.;\n"
"}}\n"
"\n"
"void HGradTetI0_basisAt(in vec3 param, out float basis[1])\n"
"{{\n"
"  HGradTetC0_basisAt(param, basis);\n"
"}}\n"
"\n"
"void HGradTetI0_basisGradientAt(in vec3 xx, out float basisGradient[3])\n"
"{{\n"
"  HGradTetC0_basisGradientAt(xx, basisGradient);\n"
"}}\n"
"#endif /* BASIS_HGradTetI0 || BASIS_HGradTetC0 */\n"
"\n"
"#ifdef BASIS_HGradTetC1\n"
"void HGradTetC1_basisAt(in vec3 param, out float basis[4])\n"
"{{\n"
"  basis[0] = 1.0 - param.x - param.y - param.z;\n"
"  basis[1] = param.x;\n"
"  basis[2] = param.y;\n"
"  basis[3] = param.z;\n"
"}}\n"
"\n"
"void HGradTetC1_basisGradientAt(in vec3 xx, out float basisGradient[12]) // 4 * 3\n"
"{{\n"
"  basisGradient[ 0] = -1.;\n"
"  basisGradient[ 1] = -1.;\n"
"  basisGradient[ 2] = -1.;\n"
"\n"
"  basisGradient[ 3] = +1.;\n"
"  basisGradient[ 4] =  0.;\n"
"  basisGradient[ 5] =  0.;\n"
"\n"
"  basisGradient[ 6] =  0.;\n"
"  basisGradient[ 7] = +1.;\n"
"  basisGradient[ 8] =  0.;\n"
"\n"
"  basisGradient[ 9] =  0.;\n"
"  basisGradient[10] =  0.;\n"
"  basisGradient[11] = +1.;\n"
"}}\n"
"#endif /* BASIS_HGradTetC1 */\n"
"\n"
"#ifdef BASIS_HCurlTetI1\n"
"void HCurlTetI1_basisAt(in vec3 param, out float basis[18]) // 6 * 3\n"
"{{\n"
"  basis[0 * 3 + 0] = 1.0 - param.y - param.z;\n"
"  basis[0 * 3 + 1] = param.x;\n"
"  basis[0 * 3 + 2] = param.x;\n"
"\n"
"  basis[1 * 3 + 0] =-param.y;\n"
"  basis[1 * 3 + 1] = param.x;\n"
"  basis[1 * 3 + 2] = 0.0;\n"
"\n"
"  basis[2 * 3 + 0] = -param.y;\n"
"  basis[2 * 3 + 1] = -1.0 + param.x + param.z;\n"
"  basis[2 * 3 + 2] = -param.y;\n"
"\n"
"  basis[3 * 3 + 0] = param.z;\n"
"  basis[3 * 3 + 1] = param.z;\n"
"  basis[3 * 3 + 2] = 1.0 - param.x - param.y;\n"
"\n"
"  basis[4 * 3 + 0] =-param.z;\n"
"  basis[4 * 3 + 1] = 0.0;\n"
"  basis[4 * 3 + 2] = param.x;\n"
"\n"
"  basis[5 * 3 + 0] = 0.0;\n"
"  basis[5 * 3 + 1] =-param.z;\n"
"  basis[5 * 3 + 2] = param.y;\n"
"}}\n"
"#endif /* BASIS_HCurlTetI1 */\n"
"\n"
"#ifdef BASIS_HDivTetI1\n"
"void HDivTetI1_basisAt(in vec3 param, out float basis[12]) // 4 * 3\n"
"{{\n"
"  basis[0 * 3 + 0] = 2.0*param.x;\n"
"  basis[0 * 3 + 1] = 2.0*(param.y - 1.0);\n"
"  basis[0 * 3 + 2] = 2.0*param.z;\n"
"\n"
"  basis[1 * 3 + 0] = 2.0*param.x;\n"
"  basis[1 * 3 + 1] = 2.0*param.y;\n"
"  basis[1 * 3 + 2] = 2.0*param.z;\n"
"\n"
"  basis[2 * 3 + 0] = 2.0*(param.x - 1.0);\n"
"  basis[2 * 3 + 1] = 2.0*param.y;\n"
"  basis[2 * 3 + 2] = 2.0*param.z;\n"
"\n"
"  basis[3 * 3 + 0] = 2.0*param.x;\n"
"  basis[3 * 3 + 1] = 2.0*param.y;\n"
"  basis[3 * 3 + 2] = 2.0*(param.z - 1.0);\n"
"}}\n"
"#endif /* BASIS_HDivTetI1 */\n"
"\n"
"#ifdef SHAPE_tetrahedron\n"
"int tetrahedron_axisPermutationForSide(in int side)\n"
"{{\n"
"  switch(side)\n"
"  {{\n"
"  case 0: return 2; // (+r, +t) = -S\n"
"  case 1: return 6; // (-r+s,-r-s+t) = +RST\n"
"  case 2: return 0; // (-s, +t) = -R\n"
"  case 3: return 4; // (-r, -s) = -T\n"
"  }}\n"
"}}\n"
"\n"
"vec3 tetrahedron_normalToSideAt(in int sideId, in float shapeVals[{ShapeCoeffPerCell}], in vec3 rst, in vec3 cameraVector)\n"
"{{\n"
"  vec3 dxdr;\n"
"  vec3 dxds;\n"
"  vec3 dxdt;\n"
"  vec3 nn;\n"
"  shapeGradientAt(rst, shapeVals, dxdr, dxds, dxdt);\n"
"  switch (sideId)\n"
"  {{\n"
"  // Triangles\n"
"  case  0: nn = cross(dxdr, dxdt); break; // -S\n"
"  case  1: nn = cross(-dxdr + dxds, dxdt - dxdr); break; // +RS\n"
"  case  2: nn = cross(dxdt, dxds); break; // -R\n"
"  case  3: nn = cross(dxds, dxdr); break; // -T\n"
"  // Edges\n"
"  case  4: nn = -dxds - dxdt; break;\n"
"  case  5: nn =  dxdr + dxds - dxdt; break;\n"
"  case  6: nn = -dxdr - dxdt; break;\n"
"  case  7: nn = -dxdr - dxds; break;\n"
"  case  8: nn =  dxdr - dxds + dxdt; break;\n"
"  case  9: nn = -dxdr + dxds + dxdt; break;\n"
"  // Vertices\n"
"  case 10: nn = -dxdr - dxds - dxdt; break;\n"
"  case 11: nn =  dxdr - dxds - dxdt; break;\n"
"  case 12: nn =  dxdr + dxds - dxdt; break;\n"
"  case 13: nn = -dxdr - dxds + dxdt; break;\n"
"  default: nn = cameraVector; break;\n"
"  }}\n"
"  return normalize(nn);\n"
"}}\n"
"\n"
"void NoneTetI0_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"void NoneTetC1_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"#endif /* SHAPE_tetrahedron */\n"
"\n"
"/** Quadrilateral function space interpolation */\n"
"#ifdef BASIS_HGradQuadI0\n"
"void HGradQuadI0_basisAt(in vec3 param, out float basis[1])\n"
"{{\n"
"  basis[0] = 1.0;\n"
"}}\n"
"\n"
"void HGradQuadI0_basisGradientAt(in vec3 param, out float basisGradient[3])\n"
"{{\n"
"  basisGradient[0] = 0.;\n"
"  basisGradient[1] = 0.;\n"
"  basisGradient[2] = 0.;\n"
"}}\n"
"#endif /* BASIS_HGradQuadI0 */\n"
"\n"
"#ifdef BASIS_HGradQuadC1\n"
"void HGradQuadC1_basisAt(in vec3 param, out float basis[4])\n"
"{{\n"
"  basis[0] = (1.0 - param.x)*(1.0 - param.y)/4.0;\n"
"  basis[1] = (1.0 + param.x)*(1.0 - param.y)/4.0;\n"
"  basis[2] = (1.0 + param.x)*(1.0 + param.y)/4.0;\n"
"  basis[3] = (1.0 - param.x)*(1.0 + param.y)/4.0;\n"
"}}\n"
"\n"
"void HGradQuadC1_basisGradientAt(in vec3 param, out float basisGradient[12]) // 4 * 3\n"
"{{\n"
"  basisGradient[ 0] =  -(1.0 - param.y)/4.0;\n"
"  basisGradient[ 1] =  -(1.0 - param.x)/4.0;\n"
"  basisGradient[ 2] =    0.0;\n"
"\n"
"  basisGradient[ 3] =   (1.0 - param.y)/4.0;\n"
"  basisGradient[ 4] =  -(1.0 + param.x)/4.0;\n"
"  basisGradient[ 5] =    0.0;\n"
"\n"
"  basisGradient[ 6] =   (1.0 + param.y)/4.0;\n"
"  basisGradient[ 7] =   (1.0 + param.x)/4.0;\n"
"  basisGradient[ 8] =    0.0;\n"
"\n"
"  basisGradient[ 9] =  -(1.0 + param.y)*(1.0 - param.z)/4.0;\n"
"  basisGradient[10] =   (1.0 - param.x)*(1.0 - param.z)/4.0;\n"
"  basisGradient[11] =    0.0;\n"
"}}\n"
"#endif /* BASIS_HGradQuadC1 */\n"
"\n"
"#ifdef SHAPE_quadrilateral\n"
"int quadrilateral_axisPermutationForSide(in int side)\n"
"{{\n"
"  switch(side)\n"
"  {{\n"
"  case -1: return 5; // (+r, +s) = +T (rendering the cell itself)\n"
"  case 0: return 2; // (+r, +t) = -S\n"
"  case 1: return 1; // (+s, +t) = +R\n"
"  case 2: return 3; // (-r, +t) = +S\n"
"  case 3: return 0; // (-s, +t) = -R\n"
"  // case 4: return 4; // (-r, -s) = -T\n"
"  // case 5: return 5; // (+r, +s) = +T\n"
"  // case 1: return 6; // (-r+s,-r-s+t) = +RST\n"
"  // case 2: return 0; // (-s, +t) = -R\n"
"  // case 3: return 4; // (-r, -s) = -T\n"
"  }}\n"
"}}\n"
"\n"
"vec3 quadrilateral_normalToSideAt(in int sideId, in float shapeVals[{ShapeCoeffPerCell}], in vec3 rst, in vec3 cameraVector)\n"
"{{\n"
"  vec3 dxdr;\n"
"  vec3 dxds;\n"
"  vec3 dxdt; // Note this should always be zero.\n"
"  vec3 nn;\n"
"  shapeGradientAt(rst, shapeVals, dxdr, dxds, dxdt);\n"
"  switch (sideId)\n"
"  {{\n"
"  // Self\n"
"  case -1: nn = cross(dxdr, dxds); break; // +T\n"
"  // Edges\n"
"  case  0: nn = -dxds; break; // -S\n"
"  case  1: nn =  dxdr; break; // +R\n"
"  case  2: nn =  dxds; break; // +S\n"
"  case  3: nn = -dxdr; break; // -R\n"
"  // Vertices\n"
"  case  4: nn = -dxdr - dxds; break;\n"
"  case  5: nn =  dxdr - dxds; break;\n"
"  case  6: nn =  dxdr + dxds; break;\n"
"  case  7: nn = -dxdr + dxds; break;\n"
"  default: nn = cameraVector; break;\n"
"  }}\n"
"  return normalize(nn);\n"
"}}\n"
"void NoneQuadI0_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"void NoneQuadC1_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"#endif /* SHAPE_quadrilateral */\n"
"\n"
"/** Triangle function space interpolation */\n"
"#ifdef BASIS_HGradTriI0\n"
"void HGradTriI0_basisAt(in vec3 param, out float basis[1])\n"
"{{\n"
"  basis[0] = 1.;\n"
"}}\n"
"\n"
"void HGradTriI0_basisGradientAt(in vec3 xx, out float basisGradient[3])\n"
"{{\n"
"  basisGradient[0] = 0.;\n"
"  basisGradient[1] = 0.;\n"
"  basisGradient[2] = 0.;\n"
"}}\n"
"#endif /* BASIS_HGradTriI0 */\n"
"\n"
"#ifdef BASIS_HGradTriC1\n"
"void HGradTriC1_basisAt(in vec3 param, out float basis[3])\n"
"{{\n"
"  basis[0] = 1.0 - param.x - param.y;\n"
"  basis[1] = param.x;\n"
"  basis[2] = param.y;\n"
"}}\n"
"\n"
"void HGradTriC1_basisGradientAt(in vec3 xx, out float basisGradient[9]) // 3 * 3\n"
"{{\n"
"  basisGradient[0] = -1.;\n"
"  basisGradient[1] = -1.;\n"
"  basisGradient[2] =  0.;\n"
"\n"
"  basisGradient[3] = +1.;\n"
"  basisGradient[4] =  0.;\n"
"  basisGradient[5] =  0.;\n"
"\n"
"  basisGradient[6] =  0.;\n"
"  basisGradient[7] = +1.;\n"
"  basisGradient[8] =  0.;\n"
"}}\n"
"#endif /* BASIS_HGradTriC1 */\n"
"\n"
"#ifdef BASIS_HCurlTriI1\n"
"void HCurlTriI1_basisAt(in vec3 param, out float[3 * 3] basis, in vec3 node[3])\n"
"{{\n"
"  vec3 edge0 = node[1] - node[0];\n"
"  vec3 edge1 = node[2] - node[1];\n"
"  vec3 edge2 = node[2] - node[0];\n"
"  vec3 edge3 = node[3] - node[0];\n"
"  vec3 edge4 = node[3] - node[1];\n"
"  vec3 edge5 = node[3] - node[2];\n"
"\n"
"  float rr = param.x;\n"
"  float ss = param.y;\n"
"  float tt = param.z;\n"
"  float uu = 1.0 - param.x - param.y - param.z;\n"
"\n"
"  basis[ 0] = rr * edge0.x;\n"
"  basis[ 1] = rr * edge0.y;\n"
"  basis[ 2] = rr * edge0.z;\n"
"\n"
"  basis[ 3] = rr * edge0.x;\n"
"  basis[ 4] = rr * edge0.y;\n"
"  basis[ 5] = rr * edge0.z;\n"
"\n"
"  basis[ 6] = rr * edge0.x;\n"
"  basis[ 7] = rr * edge0.y;\n"
"  basis[ 8] = rr * edge0.z;\n"
"}}\n"
"#endif /* BASIS_HCurlTriI1 */\n"
"\n"
"#ifdef SHAPE_triangle\n"
"int triangle_axisPermutationForSide(in int side)\n"
"{{\n"
"  switch(side)\n"
"  {{\n"
"  case -1: return 5; // (+r, +s) = +T (normal to triangle itself)\n"
"  case 0: return 2; // (+r, +t) = -S\n"
"  case 1: return 6; // (-r+s,-r-s) = +RS (T)\n"
"  case 2: return 0; // (-s, +t) = -R\n"
"  // case 3: return 4; // (-r, -s) = -T\n"
"  }}\n"
"}}\n"
"\n"
"vec3 triangle_normalToSideAt(in int sideId, in float shapeVals[{ShapeCoeffPerCell}], in vec3 rst, in vec3 cameraVector)\n"
"{{\n"
"  vec3 dxdr;\n"
"  vec3 dxds;\n"
"  vec3 dxdt; // This will always be zero.\n"
"  vec3 nn;\n"
"  shapeGradientAt(rst, shapeVals, dxdr, dxds, dxdt);\n"
"  switch (sideId)\n"
"  {{\n"
"  // Self\n"
"  case -1: nn = cross(dxdr, dxds); break; // +T\n"
"  // Edges\n"
"  case  0: nn = -dxds; break; // -S\n"
"  case  1: nn =  dxdr + dxds; break; // +RS\n"
"  case  2: nn = -dxdr; break; // -R\n"
"  // Vertices\n"
"  case  4: nn = -dxdr - dxds; break;\n"
"  case  5: nn =  dxdr - 0.5 * dxds; break;\n"
"  case  6: nn = -0.5 * dxdr + dxds; break;\n"
"  default: nn = cameraVector; break;\n"
"  }}\n"
"  return normalize(nn);\n"
"}}\n"
"\n"
"void NoneTriI0_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"void NoneTriC1_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"#endif /* SHAPE_triangle */\n"
"\n"
"/** Edge function space interpolation */\n"
"#ifdef BASIS_HGradEdgeI0\n"
"void HGradEdgeI0_basisAt(in vec3 param, out float basis[1])\n"
"{{\n"
"  basis[0] = 1.0;\n"
"}}\n"
"\n"
"void HGradEdgeI0_basisGradientAt(in vec3 xx, out float basisGradient[3])\n"
"{{\n"
"  basisGradient[0] = 0.;\n"
"  basisGradient[1] = 0.;\n"
"  basisGradient[2] = 0.;\n"
"}}\n"
"#endif /* BASIS_HGradEdgeI0 */\n"
"\n"
"#ifdef BASIS_HGradEdgeC1\n"
"void HGradEdgeC1_basisAt(in vec3 param, out float basis[2])\n"
"{{\n"
"  basis[0] = 1.0 - param.x;\n"
"  basis[1] = param.x;\n"
"}}\n"
"\n"
"void HGradEdgeC1_basisGradientAt(in vec3 xx, out float basisGradient[6]) // 2 * 3\n"
"{{\n"
"  basisGradient[0] = -1.;\n"
"  basisGradient[1] =  0.;\n"
"  basisGradient[2] =  0.;\n"
"\n"
"  basisGradient[3] = +1.;\n"
"  basisGradient[4] =  0.;\n"
"  basisGradient[5] =  0.;\n"
"}}\n"
"#endif /* BASIS_HGradEdgeC1 */\n"
"\n"
"#ifdef SHAPE_edge\n"
"int edge_axisPermutationForSide(in int side)\n"
"{{\n"
"  if (side < 0) {{ return 7; }} // Camera-facing normal.\n"
"  return side; // side 0 = -R (0), side 1 = +R (1)\n"
"}}\n"
"\n"
"vec3 edge_normalToSideAt(in int sideId, in float shapeVals[{ShapeCoeffPerCell}], in vec3 rst, in vec3 cameraVector)\n"
"{{\n"
"  float eps = 1.19209e-07;\n"
"  vec3 dxdr;\n"
"  vec3 dxds; // This will always be zero.\n"
"  vec3 dxdt; // This will always be zero.\n"
"  vec3 nn;\n"
"  shapeGradientAt(rst, shapeVals, dxdr, dxds, dxdt);\n"
"  switch (sideId)\n"
"  {{\n"
"  // Self\n"
"  case -1:\n"
"    nn = cross(dxdr, vec3(0, 1, 0));\n"
"    if (length(nn) < eps)\n"
"    {{\n"
"      nn = cross(dxdr, vec3(0, 0, 1));\n"
"    }}\n"
"    break;\n"
"  // Vertices\n"
"  case  0: nn = -dxdr; break; // -R\n"
"  case  1: nn =  dxdr; break; // +R\n"
"  default: nn = cameraVector; break;\n"
"  }}\n"
"  return normalize(nn);\n"
"}}\n"
"\n"
"void NoneEdgeI0_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"void NoneEdgeC1_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"#endif /* SHAPE_edge */\n"
"\n"
"/** Vertex function space interpolation */\n"
"#ifdef BASIS_HGradVertI0\n"
"void HGradVertI0_basisAt(in vec3 param, out float basis[0])\n"
"{{\n"
"  basis[0] = 1.0;\n"
"}}\n"
"\n"
"void HGradVertI0_basisGradientAt(in vec3 xx, out float basisGradient[3])\n"
"{{\n"
"  basisGradient[0] = 0.;\n"
"  basisGradient[1] = 0.;\n"
"  basisGradient[2] = 0.;\n"
"}}\n"
"#endif /* BASIS_HGradVertI0 */\n"
"\n"
"#ifdef BASIS_HGradVertC1\n"
"// TODO: Eliminate this as it makes no sense to have a linear basis over a null space.\n"
"void HGradVertC1_basisAt(in vec3 param, out float basis[1])\n"
"{{\n"
"  basis[0] = 1.0;\n"
"}}\n"
"\n"
"void HGradVertC1_basisGradientAt(in vec3 xx, out float basisGradient[3])\n"
"{{\n"
"  basisGradient[0] = 0.;\n"
"  basisGradient[1] = 0.;\n"
"  basisGradient[2] = 0.;\n"
"}}\n"
"#endif /* BASIS_HGradVertC1 */\n"
"\n"
"#ifdef SHAPE_vertex\n"
"int vertex_axisPermutationForSide(in int side)\n"
"{{\n"
"  return 7; // Camera-facing normal\n"
"}}\n"
"\n"
"vec3 vertex_normalToSideAt(in int sideId, in float shapeVals[{ShapeCoeffPerCell}], in vec3 rst, in vec3 cameraVector)\n"
"{{\n"
"  vec3 nn;\n"
"  nn = cameraVector;\n"
"  return normalize(nn);\n"
"}}\n"
"\n"
"void NoneVertI0_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"void NoneVertC1_basisAt(in vec3 param, out float[1] basis) {{ }}\n"
"#endif /* SHAPE_vertex */\n"
"";
